﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Description;
using Microsoft.ServiceBus.Messaging;
using Microsoft.ServiceBus.Messaging.Filters;
using System.Diagnostics;

namespace CreateInquiry
{



    //TODO: All of the code for working with the service bus needs to be factored out and into its own 
    //      container.
    //      
    //TODO: Genericize methods for Queue and Topics
    //
    public partial class RequestResponse : Form
    {
        public RequestResponse()
        {
            InitializeComponent();
        }

        private DataSet dsSet = new DataSet();

        //TODO: These belong in config
        private const string sbNamespace = "jofultz";
        private const string issuerName = "owner";
        private const string issuerKey = "";
        private const string topicName = "Inventory Check";
        private const string subscriptionName = "InventoryQuerySubscription";
        private const string returnQueueName = "InventoryResponse";
        private Queue returnQ = null;
        private Timer checkQTimer = new Timer();

        private List<BrokeredMessage> Inquiries = new List<BrokeredMessage>();
        private InquiryResponseList InquiryResponses = new InquiryResponseList();


        private Uri _SbUri = default(Uri);
        private Uri ServiceBusUri
        {
            get
            {
                if (_SbUri == default(Uri))
                {
                    _SbUri = ServiceBusEnvironment.CreateServiceUri("sb", sbNamespace, String.Empty);
                }
                return _SbUri;
            }

        }

        private SharedSecretCredential _SharedSecretCred = default(SharedSecretCredential);
        private SharedSecretCredential ServiceBusCredential
        {
            get
            {
                if (_SharedSecretCred == default(SharedSecretCredential))
                {
                    _SharedSecretCred = TransportClientCredentialBase.CreateSharedSecretCredential(issuerName, issuerKey);
                }
                return _SharedSecretCred;
            }
        }

        private ServiceBusNamespaceClient _sbnsclient = default(ServiceBusNamespaceClient);
        private ServiceBusNamespaceClient ServiceBusNSClient
        {
            get
            {
                if (_sbnsclient == default(ServiceBusNamespaceClient))
                {
                    _sbnsclient = new ServiceBusNamespaceClient(this.ServiceBusUri, this.ServiceBusCredential);
                }
                return _sbnsclient;
            }
        }

        private MessagingFactory _msgFactory = null;
        private MessagingFactory MsgFactory
        {
            get { 
                if (_msgFactory == null)
                {
                    _msgFactory = MessagingFactory.Create(ServiceBusUri,ServiceBusCredential);
                }
                return _msgFactory;
            }
        }

        private QueueClient _qClient = null;
        private QueueClient QClient 
        {
            get{
                if(_qClient == null)
                {
                    _qClient = MsgFactory.CreateQueueClient(returnQueueName);
                }
                return _qClient;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            
            dsSet.ReadXml(Application.StartupPath + "\\StoreList.xml");
            cboStore.DataSource = dsSet.Tables["Store"];
            cboStore.DisplayMember = "id";
            cboStore.ValueMember = "region";

            checkQTimer.Interval = 3000;
            checkQTimer.Tick += new EventHandler(checkQTimer_Tick);
            
        }

        void checkQTimer_Tick(object sender, EventArgs e)
        {
            MessageReceiver receiver = QClient.CreateReceiver(ReceiveMode.ReceiveAndDelete );
            BrokeredMessage NewMessage = null;
            while (receiver.TryReceive(TimeSpan.FromSeconds(1), out NewMessage))
            {
                
                InquiryResponse resp = new InquiryResponse();
                resp.CorrelationID = NewMessage.CorrelationId;
                resp.Message = NewMessage;
                InquiryResponses.Add(resp);
            }
       }

        public void CreateTopic(String topicName)
        {
            try
            {
                Topic newTopic = ServiceBusNSClient.CreateTopic(topicName);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        public void EnumerateTopics()
        {
             try
            {

                IEnumerable<Topic> topics = ServiceBusNSClient.GetTopics();
                
                foreach (Topic topic in topics)
                {
                    String path = topic.Path;
                    Console.WriteLine(path);

                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

        }


        private void cmdCheck_Click(object sender, EventArgs e)
        {
            cmdCheck.Enabled = false;
            lblStatus.Text = "Sending ... ";
            //assign data values
            InventoryQueryData data = new InventoryQueryData();
            data.Sku = txtSKU.Text;
            data.RequesterID = cboStore.Text;
            data.Region = this.cboStore.SelectedValue.ToString();
            
            MessagingFactory msgFactory = MessagingFactory.Create(this.ServiceBusUri, this.ServiceBusCredential);
            TopicClient topicClient = msgFactory.CreateTopicClient(topicName);
            MessageSender MsgSender = topicClient.CreateSender();
            
            BrokeredMessage msg = BrokeredMessage.CreateMessage(data);
            //add props to message for filtering
            msg.Properties["Region"] = data.Region;
            msg.Properties["RequesterID"] = data.RequesterID;

            //set properties for message
            msg.TimeToLive = TimeSpan.FromSeconds(30);
            msg.ReplyTo = returnQueueName;
            msg.CorrelationId = Guid.NewGuid().ToString();
            
            //check if-exists and create response queue
            try
            {
                returnQ = this.ServiceBusNSClient.GetQueue(returnQueueName);
            }
            catch (System.Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }

            if (returnQ == null)
            {
                returnQ = this.ServiceBusNSClient.CreateQueue(returnQueueName);
            }

            checkQTimer.Enabled = true;

            //send message
            MsgSender.Send(msg);
            
            Inquiries.Add(msg);
            this.lbRequests.Items.Add(msg.CorrelationId);

            lblStatus.Text = msg.MessageId + " sent!";
            cmdCheck.Enabled = true;
        }

        private void lbRequests_SelectedIndexChanged(object sender, EventArgs e)
        {
            string correlationId = lbRequests.SelectedItem.ToString();

            List<InquiryResponse> CorrelatedList = InquiryResponses.FindAll(
                                                delegate(InquiryResponse resp)
                                                {
                                                    return resp.CorrelationID == correlationId;
                                                });
            dgvResponses.Rows.Clear();

            foreach(InquiryResponse resp in CorrelatedList)
            {
                BrokeredMessage msg = resp.Message;
                CreateInquiry.InventoryQueryData Inquiry = msg.GetBody<InventoryQueryData>();
                dgvResponses.Rows.Add( new object[5]{msg.MessageId, msg.CorrelationId, Inquiry.ResponderID, Inquiry.Sku, Inquiry.Count});
            }
        }

    }
    public class InquiryResponse
    {
        public string CorrelationID;
        public BrokeredMessage Message;
    }

    public class InquiryResponseList : List<InquiryResponse>
    {

    }
}
