﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.ServiceBus.Messaging;
using Microsoft.ServiceBus.Messaging.Filters;
using Microsoft.ServiceBus.Description;
using Microsoft.ServiceBus;
using System.Diagnostics;

namespace TopicMonitor
{

    public delegate void MessageReceivedDelegate(object sender, MessagingEventArgs e);
    public delegate void ThreadControlDelegate(string StoreID, ThreadControlEnum ControlCommand);

    public enum ThreadControlEnum
    {
        Start=1,
        Stop=0,
        Pause=2
    }

    public class MessagingEventArgs : EventArgs
    {
        public MessagingEventArgs(object data)
        {
            MessageData = data;
        }
        public object MessageData;
    }

    //TODO: These belong in config
    public static class Constants
    {
        public const string sbNamespace = "jofultz";
        public const string issuerName = "owner";
        public const string issuerKey = "";
        public const string topicName = "Inventory Check";
        public const string SubscriptionNameRoot = "InventoryQuerySubscription";
    }

    public static class EventContainer
    {
        public static event MessageReceivedDelegate MessageReceivedEvent;
        public static event ThreadControlDelegate ThreadControlEvent;

        public static void RaiseThreadControl(string StoreID, ThreadControlEnum ControlCommand)
        {
            ThreadControlEvent(StoreID, ControlCommand);
        }

        //TODO:  need to fix to pass the brokered message as the only data
        public static void RaiseMessageReceived(object data)
        {
            MessageReceivedEvent(data, new MessagingEventArgs(data));
        }
    }

    public class MonitorThread
    {
        private SharedSecretCredential credential = default(SharedSecretCredential);
        private Uri sbUri = default(Uri);
        private ServiceBusNamespaceClient sbNSClient = default(ServiceBusNamespaceClient);
        private MessagingFactory msgFactory = default(MessagingFactory);

        private System.Timers.Timer MonitorTimer = new System.Timers.Timer();
        private string _StoreID = default(string);
        private bool _MonitorOn = false;

        private string SubscriptionName
        {
            get {return Constants.SubscriptionNameRoot + StoreID;}
        }


        private bool MonitorOn
        {
            get { return _MonitorOn; }
            set { _MonitorOn = value; }
        }

        public string StoreID
        {
            get{return _StoreID;}
            set
            {
                _StoreID = value;
            }
        }

        public string Region = default(string);

        public MonitorThread()
        {
            EventContainer.ThreadControlEvent += new ThreadControlDelegate(EventContainer_ThreadControlEvent);

            credential = TransportClientCredentialBase.CreateSharedSecretCredential(Constants.issuerName, Constants.issuerKey);
            sbUri = ServiceBusEnvironment.CreateServiceUri("sb", Constants.sbNamespace, String.Empty);
            sbNSClient = new ServiceBusNamespaceClient(sbUri, credential);
            


            MonitorTimer.Elapsed += new System.Timers.ElapsedEventHandler(MonitorTimer_Tick);
            MonitorTimer.Interval = 3000;

            msgFactory = MessagingFactory.Create(sbUri, credential);

        }

        void EventContainer_ThreadControlEvent(string StoreID, ThreadControlEnum ControlCommand)
        {
            switch (ControlCommand)
            {
                case ThreadControlEnum.Start:
                    MonitorOn = true;
                    break;
                case ThreadControlEnum.Stop:
                    MonitorOn = false;
                    this.MonitorTimer.Stop();
                    this.MonitorTimer = null;

                    break;

                case ThreadControlEnum.Pause:
                    MonitorOn = false;
                    break;
            }
        }

        public void MonitorStart()
        {
            Topic invTopic = sbNSClient.GetTopic(Constants.topicName);

            try
            {
                IEnumerable<Subscription> subs = invTopic.GetSubscriptions();

                foreach (Subscription sub in subs)
                {
                    if (sub.Name.Equals(this.SubscriptionName))
                    {
                        invTopic.RemoveSubscription(sub.Name);
                        Debug.WriteLine(sub.Name + " - removed");
                    }
                }



            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }

            try
            {

                SqlFilterExpression RegionFilter = new SqlFilterExpression("Region = '" + this.Region + "' AND RequesterID <> '" + this.StoreID + "'");
                Subscription sub = invTopic.AddSubscription(SubscriptionName, RegionFilter);


            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }

            MonitorTimer.Start();            

        }

        void MonitorTimer_Tick(object sender, EventArgs e)
        {
            MonitorTimer.Stop();
         
            SubscriptionClient subClient = msgFactory.CreateSubscriptionClient(Constants.topicName, SubscriptionName);
            MessageReceiver msgReceiver = subClient.CreateReceiver(ReceiveMode.ReceiveAndDelete);

            msgReceiver.Open();

            bool recvSuccess = false;
            do
            {
                BrokeredMessage NewMsg;
                recvSuccess = msgReceiver.TryReceive(TimeSpan.FromSeconds(3), out NewMsg);
                if (recvSuccess)
                {
                    CreateInquiry.InventoryQueryData qryData = NewMsg.GetBody<CreateInquiry.InventoryQueryData>();

                    NewMsg.Properties.Add("ResponderId", this.StoreID);
                    EventContainer.RaiseMessageReceived(NewMsg);

                }

            } while (recvSuccess);

            if (MonitorTimer != null) { MonitorTimer.Start(); }


        }
    }
}
