﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.ServiceBus.Messaging;
using Microsoft.ServiceBus.Messaging.Filters;
using Microsoft.ServiceBus.Description;
using Microsoft.ServiceBus;
using System.Diagnostics;

namespace TopicMonitor
{
    public partial class TopicMonitorAndResponder : Form
    {
        public TopicMonitorAndResponder()
        {
            InitializeComponent();
        }

        private DataSet dsSet = new DataSet();
        private bool MonitorOn = false;
                
        private Dictionary<string, bool> MonitorIdTracking = new Dictionary<string, bool>();
        private Dictionary<string, BrokeredMessage> ReceivedMessages = new Dictionary<string, BrokeredMessage>();


        private void Form1_Load(object sender, EventArgs e)
        {
            
            dsSet.ReadXml(Application.StartupPath + "\\StoreList.xml");
            cboStore.DataSource = dsSet.Tables["Store"];
            cboStore.DisplayMember = "id";
            cboStore.ValueMember = "region";

            this.cboStore.SelectedIndexChanged += new System.EventHandler(this.cboStore_SelectedIndexChanged);
            EventContainer.MessageReceivedEvent += new MessageReceivedDelegate(EventContainer_MessageReceivedEvent);



        }

        void EventContainer_MessageReceivedEvent(object sender, MessagingEventArgs e)
        {
            BrokeredMessage msg = (BrokeredMessage)e.MessageData;
            this.ReceivedMessages.Add(msg.MessageId, msg);


            UpdateUIElements(msg.MessageId, msg);

        }


        delegate void UpdateUIElementsDelegate(string text, BrokeredMessage data);

        private void UpdateUIElements(string text, BrokeredMessage data)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.dgInquiries.InvokeRequired)
            {
                UpdateUIElementsDelegate callback = new UpdateUIElementsDelegate(UpdateUIElements);
                this.Invoke(callback, new object[] { text, data });
            }
            else
            {

                var RequestObject = data.GetBody<CreateInquiry.InventoryQueryData>();
                RequestObject.ResponderID = data.Properties["ResponderId"].ToString();

                object [] values = new object[4]{data.MessageId, data.CorrelationId, RequestObject.RequesterID, RequestObject.Sku};
               
                dgInquiries.Rows.Add(values);
               dgInquiries.Refresh();
            }
        }

        //TODO:  Stop monitor doesn't work properly; need to debug
        private void cmdStartMonitor_Click(object sender, EventArgs e)
        {
            if (cmdStartMonitor.Text == "Stop Monitor")
            {

                cmdStartMonitor.Text = "Start Monitor";

                EventContainer.RaiseThreadControl(cboStore.Text, ThreadControlEnum.Stop);
                MonitorIdTracking.Remove(cboStore.Text);

                MonitorOn = false;

            }
            else
            {
                //TODO: start thread and connect events
                //TODO: add tracking value to dictionary
                this.MonitorIdTracking.Add(cboStore.Text, true);

                cmdStartMonitor.Text = "Stop Monitor";

                MonitorThread mt = new MonitorThread();
                mt.StoreID = cboStore.Text;
                mt.Region = cboStore.SelectedValue.ToString();
                System.Threading.ThreadStart tstart = new System.Threading.ThreadStart(mt.MonitorStart);
                System.Threading.Thread newThread = new System.Threading.Thread(tstart);
                newThread.Start();

                this.MonitorOn = true;
 
            }


        }


        private void cboStore_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {

                ComboBox cbo = (ComboBox)sender;
                bool CurrentlyMonitoring;
                this.MonitorIdTracking.TryGetValue(cbo.SelectedText, out CurrentlyMonitoring);
                
                if (CurrentlyMonitoring)
                {
                    MonitorOn = CurrentlyMonitoring;
                    this.cmdStartMonitor.Text = "Stop Monitor";

                }
                else
                {
                    MonitorOn = CurrentlyMonitoring;
                    this.cmdStartMonitor.Text = "Start Monitor";
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }

        }

        private void cmdSendResponse_Click(object sender, EventArgs e)
        {
            string SelectedMsgId = dgInquiries.SelectedRows[0].Cells[0].Value.ToString();

            //create response message
            BrokeredMessage msg = ReceivedMessages[SelectedMsgId];
            CreateInquiry.InventoryQueryData inquiry = msg.GetBody<CreateInquiry.InventoryQueryData>();

            inquiry.ResponderID = this.cboStore.Text;
            inquiry.Count = int.Parse(this.txtQuantity.Text);

            BrokeredMessage ResponseMsg =  BrokeredMessage.CreateMessage(inquiry);
            ResponseMsg.CorrelationId = msg.CorrelationId;

            //send message
            SharedSecretCredential credential = TransportClientCredentialBase.CreateSharedSecretCredential(Constants.issuerName, Constants.issuerKey);
            Uri sbUri = ServiceBusEnvironment.CreateServiceUri("sb", Constants.sbNamespace, String.Empty);

            MessagingFactory Factory =  MessagingFactory.Create(sbUri, credential);
            QueueClient QClient = Factory.CreateQueueClient(msg.ReplyTo);
            MessageSender Sender = QClient.CreateSender();

            Sender.Send(ResponseMsg);


        }

        private void dgInquiries_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {

        }
    }
}
